#!/usr/bin/python3
# coding:utf-8

# Copyright 2018 ISB Corp. <http://www.isb.co.jp/>
# All rights reserved.

import sys
import signal
import binascii
from time import sleep

from isb_wsn_serial import WiSunSerial
from isb_wsn_ota import WiSunOTA
import isb_wsn_serial
import isb_wsn_ota

def ota_client_main(serial, _mac):
	wsn_ota = WiSunOTA(serial, _mac)

	# Initiate OTA Client
	ret = serial.cmd_ota_client_start()
	if( ret != 0): return ret

	print('\nWaiting for OTA Update Completion...')
	while True:
		ret = wsn_ota.recv_for_client()
		if(ret == 2): break

	# Terminate OTA Client
	ret = serial.cmd_ota_client_end()
	return ret

def han_enddev_main(serial, channel, cliPass):
	# Reset Hardware
	ret = serial.cmd_com_reset()
	if( ret != 0): return ret
	# Initial settings
	init_settings = [
		0x03,		# Operation mode(0x01:PAN coordinator,0x02:Coordinator,0x03:End device)
 		0x00,		# HAN sleep function setting(0x00:Disabled,0x01:Enabled)
 		channel,	# Channel(0x04 - 0x11)
 		0x00		# Transmission power(0x00:20mW,0x01:10mW,0x02:1mW)
	]
	ret = serial.cmd_com_init(init_settings)
	if( ret != 0): return ret

	# Initiate HAN Operation 
	ret = serial.cmd_han_start(0x0)
	if( ret != 0): return ret

	# Set HAN PANA Authentication Information
	ret = serial.cmd_han_pana_setting( [], cliPass)
	if( ret != 0): return ret

	# Initiate HAN PANA 
	ret = serial.cmd_han_pana_start()
	if( ret != 0): return ret

	print('\nEnd Device started.\nWaiting for authentication completion...')
	ret = serial.wait_for_pana_auth()

	return ret

def main(ttyDev, channel, srvAddr, cliPass):
	ret = 0

	# open serial port
	wsn_ser = WiSunSerial(ttyDev)
	if(wsn_ser.isOpen() == False): return 1

	cliPass = isb_wsn_serial.input_pana_pass(cliPass)
	ret = han_enddev_main(wsn_ser, channel, cliPass)
	if(ret != 0): return ret

	srvAddr = isb_wsn_serial.input_dest_mac(srvAddr)
	ret = ota_client_main(wsn_ser, srvAddr)
	return ret

def sigint_handler(signal, frame):
	print('\nterminated by user')
	sys.exit(0)

if __name__ == '__main__':
	import argparse

	parser = argparse.ArgumentParser( description='Sample script for OTA Client' )
	parser.add_argument(
		'-s','--serialport',
		dest='port', default=None, required=True,
		help='Serial port number.\n ex: Win=\'COMx\', Linux=\'/dev/ttyUSBx\'' )
	parser.add_argument(
		'-c','--channel',
		dest='ch', default=None, required=True,
		help='Wi-SUN channel(decimal)' )
	parser.add_argument(
		'-sa','--serveraddr',
		dest='srvAddr', default=None, required=False,
		help='OTA Server Address(8byte hex string)' )
	parser.add_argument(
		'-cp','--clientpassword',
		dest='cliPass', default=None, required=False,
		help='OTA Client Password(16byte hex string)' )

	signal.signal(signal.SIGINT, sigint_handler)

	args = parser.parse_args()
	ret = main(args.port,int(args.ch), args.srvAddr, args.cliPass)

	sys.exit(ret)
