#!/usr/bin/python3
# coding:utf-8

# Copyright 2018 ISB Corp. <http://www.isb.co.jp/>
# All rights reserved.

import sys
import signal
import binascii

from isb_wsn_serial import WiSunSerial
from isb_wsn_ota import WiSunOTA
import isb_wsn_ota
import isb_wsn_serial

def print_ota_srv_menu():
	print('\nOTA Update Menu:')
	print(' Please input index')
	print('\t0. Open UDP Port({:d})'.format(isb_wsn_ota.UDP_PORT_OTA))
	print('\t1. Start OTA Mode')
	print('\t2. Get OTA Version Information')
	print('\t3. Get OTA Write BANK Information')
	print('\t4. Start OTA Write')
	print('\t5. OTA Data Write')
	print('\t6. End OTA Write')
	print('\t7. End OTA Mode')
	print('\th. show this Help')

def ota_server_menu(serial, _mac):
	wsn_ota = WiSunOTA(serial, _mac)

	print_ota_srv_menu()
	input_str = ""
	while input_str == "":
		input_str = input('>>> ');

		serial.flush_recv_buf()

		if input_str == "":
			continue
		elif input_str == "h":
			print_ota_srv_menu()
		elif input_str == "0":
			# Open UDP Port
			serial.cmd_com_open_port(isb_wsn_ota.UDP_PORT_OTA)
		elif input_str == "1":
			# Request Start OTA Mode(cmd=0x61->0x71)
			wsn_ota.otamode_start()
		elif input_str == "2":
			# Get OTA Version Information(cmd=0x68->0x78)
			wsn_ota.get_ver()
		elif input_str == "3":
			# Get OTA Write BANK Information(cmd=0x62->0x72)
			wsn_ota.get_bank_info()
		elif input_str == "4":
			# Start OTA Write(cmd=0x40->0x70)
			hexFile = isb_wsn_ota.input_hex_file(None)
			if(len(hexFile)==0): break
			print("loading hex file...")
			wsn_ota.load_fw_data(hexFile)
			wsn_ota.write_start()
		elif input_str == "5":
			# OTA Data Write
			wsn_ota.send_all_otadata()
		elif input_str == "6":
			# End OTA Write(cmd=0x45->0x75)
			wsn_ota.write_end()
		elif input_str == "7":
			# End OTA Mode(cmd=0x64->0x74)
			wsn_ota.otamode_end()
		else:
			print('unkown index')
			print_ota_srv_menu()

		# continue loop
		input_str = ""

def han_panc_main(serial, channel, panid, cliAddr, cliPass):
	# Reset Hardware
	ret = serial.cmd_com_reset()
	if( ret != 0): return ret
	# Set Initial Settings
	init_settings = [
		0x01,		# Operation mode(0x01:PAN coordinator,0x02:Coordinator,0x03:End device)
 		0x00,		# HAN sleep function setting(0x00:Disabled,0x01:Enabled)
 		channel,	# Channel(0x04 - 0x11)
 		0x00		# Transmission power(0x00:20mW,0x01:10mW,0x02:1mW)
	]
	ret = serial.cmd_com_init(init_settings)
	if( ret != 0): return ret

	# Initiate HAN Operation
	ret = serial.cmd_han_start(panid)
	if( ret != 0): return ret

	# Set HAN PANA Authentication Information
	ret = serial.cmd_han_pana_setting( cliAddr, cliPass)
	if( ret != 0): return ret

	# Initiate HAN PANA 
	ret = serial.cmd_han_pana_start()
	if( ret != 0): return ret

	# Switch HAN Acceptance Connection Mode(Normal->Initial)
	mode = 0x01 # Initial connection mode
	ret = serial.cmd_han_switch_connect_mode(mode)
	if( ret != 0): return ret

	print('\nPAN coordinator started.\nWaiting for client connection...')
	ret = serial.wait_for_connect(cliAddr)
	if( ret != 0): return ret

	# Switch HAN Acceptance Connection Mode(Initial->Normal)
	mode = 0x02 # Normal connection mode
	ret = serial.cmd_han_switch_connect_mode(mode)
	if( ret != 0): return ret

	# end function
	return ret


def main(ttyDev, channel, panid, cliAddr, cliPass):
	ret = 0

	# open serial port
	wsn_ser = WiSunSerial(ttyDev)
	if(wsn_ser.isOpen() == False): return 1

	cliAddr = isb_wsn_serial.input_dest_mac(cliAddr)
	cliPass = isb_wsn_serial.input_pana_pass(cliPass)
	ret = han_panc_main(wsn_ser, channel, panid, cliAddr, cliPass)
	if(ret != 0): return ret

	ret = ota_server_menu(wsn_ser, cliAddr)
	if(ret != 0): return ret

	# end function
	return ret

def sigint_handler(signal, frame):
	print('\nterminated by user')
	sys.exit(0)

if __name__ == '__main__':
	import argparse

	parser = argparse.ArgumentParser( description='Sample script for OTA Server' )
	parser.add_argument(
		'-s','--serialport',
		dest='port', default=None, required=True,
		help='Serial port number.\n ex: Win=\'COMx\', Linux=\'/dev/ttyUSBx\'' )
	parser.add_argument(
		'-c','--channel',
		dest='ch', default=None, required=True,
		help='Wi-SUN channel(decimal)' )
	parser.add_argument(
		'-p','--panid',
		dest='panid', default=None, required=True,
		help='Wi-SUN PAN-ID(2byte hex string)' )
	parser.add_argument(
		'-ca','--clientaddr',
		dest='cliAddr', default=None, required=False,
		help='Client Address(8byte hex string)' )
	parser.add_argument(
		'-cp','--clientpassword',
		dest='cliPass', default=None, required=False,
		help='Client Password(16byte hex string)' )

	signal.signal(signal.SIGINT, sigint_handler)

	args = parser.parse_args()
	ret = main(args.port,int(args.ch),int(args.panid, 16), args.cliAddr, args.cliPass)

	sys.exit(ret)
